# Pyromaths
# -*- coding: utf-8 -*-
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if notPopen, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

import random
from ..outils.Affichage import decimaux, decimaux_quiz

def noms_sommets(nb):
    """Renvoie nb noms de sommets"""
    (listenb, listepts) = ([], [])
    for i in range(26):
        listenb.append(i + 65)
    for i in range(nb):
        listepts.append(str(chr(listenb.pop(random.randrange(26 - i)))))
    listepts.sort()
    return tuple(listepts)

def angle(coordo):
    """Angle pour placer le nom du point en fonction du quadrant"""
    l=[]
    for coord in coordo:
        if coord[0]>=0 and coord[1]>=0:
            ang='45'
        if coord[0]>=0 and coord[1]<0:
            ang='-45'
        if coord[0]<0 and coord[1]<0:
            ang='-135'
        if coord[0]<0 and coord[1]>=0:
            ang='135'


        l.append(ang)
    return l

def place_points(coordo,points):
    """coordo: liste de coordonnées ;
    points : noms des points associés
    génère le code latex pour placer tous les points"""
    l= angle(coordo)
    text="\\pstGeonode[PointSymbol=x,PosAngle={"
    for ang in l :
        text=text+ang+','
    text=text+"},PointNameSep=0.4]"
    i=0
    while i<len(coordo):
        text=text+str(coordo[i])+"{"+str(points[i])+"}"
        i=i+1
    return text

def valide(coord,liste_coord):
    """Évite deux points trop proches l'un de l'autre"""
    rep=True
    for c in liste_coord:
        if abs(coord[0]-c[0])<=0.5 and abs(coord[1]-c[1])<=0.5:
            rep=False
            break
    return rep

def quadrant(coord,liste_coord):
    """Évite 2 points dans le même quadrant"""
    rep= True
    for c in liste_coord:
        if coord[0]*c[0]>=0 and coord[1]*c[1]>=0:
            rep=False
            break
    return rep


def coordo_pts(nb):
    """Génère une liste de nb coordonnées. Je m'arrange pour avoir des points
    sur les 4 quadrants et sur les deux axes"""
    j=0
    k=0
    l=[]
    i=0
    while i<4 and i<nb:
        j=j+1
        if j==600:
            # Correction d'un bug
            l, i, j = [], 0, 0
        a=float(random.randrange(-9,10))/2
        b=float(random.randrange(-9,10))/2
        if ((a,b) not in l) and valide((a,b),l)and quadrant((a,b),l):
            l.append((a,b))
            i=i+1
    if nb>=4:
        a=float(random.randrange(-9,10))/2
        while not valide((0,a),l):
            a=float(random.randrange(-9,10))/2
        rg=random.randrange(0,len(l)-1)
        l[rg:rg]=[(0,a)]
        i=i+1

    if nb>=5:
        b=float(random.randrange(-9,10))/2
        while not valide((b,0),l):
            b=float(random.randrange(-9,10))/2
        rg=random.randrange(0,len(l)-1)
        l[rg:rg]=[(b,0)]
        i=i+1


    while i>=6 and i<nb and i<10:
        a=float(random.randrange(-9,10))/2
        b=float(random.randrange(-9,10))/2
        j=j+1
        if j==600:
            break
        if ((a,b) not in l) and valide((a,b),l)and quadrant((a,b),l[6:i]):
            l.append((a,b))
            i=i+1
    if nb>=11 and len(l)==10:
        a=float(random.randrange(-9,10))/2
        while not valide((0,a),l):
            a=float(random.randrange(-9,10))/2
        rg=random.randrange(6,len(l)-1)
        l[rg:rg]=[(0,a)]
        i=i+1
    if nb>=11 and len(l)==11:
        b=float(random.randrange(-9,10))/2
        while not valide((b,0),l):
            b=float(random.randrange(-9,10))/2
        rg=random.randrange(6,len(l)-1)
        l[rg:rg]=[(b,0)]
        i=i+1

    while i>=12 and i<nb :
        a=float(random.randrange(-9,10))/2
        b=float(random.randrange(-9,10))/2
        k=k+1
        if k==600:
            break
        if ((a,b) not in l) and valide((a,b),l):
            l.append((a,b))
            i=i+1
    return l


def affiche_coord(coord):
    """Affiche les coordonnées des points au format LaTeX"""
    return '\\hbox{$('+decimaux(str(coord[0]), 1) + '~;~' + \
                                             decimaux(str(coord[1]), 1)+')$}'

def affiche_coord_quiz(coord):
    """Affiche les coordonnées des points au format quiz"""
    #Cambiar a NUMERICAL cuando arreglen el bug de quiz que lo da por malo, y también al final de decimaux_quiz poner 1 en lugar de 0
    quiz_txt = "$$($$" + ("{1:SHORTANSWER:%s100%s%s} $$;$$" %("%", "%", decimaux_quiz(str(coord[0]), 0)))
    quiz_txt += ("{1:SHORTANSWER:%s100%s%s} $$)$$" %("%", "%", decimaux_quiz(str(coord[1]), 0)))
    return quiz_txt

def tex_liste_co(liste_coord):
    """prend une liste de coordonnées et renvoie la liste en string prête pour latex
    #\hbox évite que les coordonnées soient coupées à l'affichage"""
    i=0
    tlc=[]
    while i < len(liste_coord):
        tlc.append(affiche_coord(liste_coord[i]))
        i=i+1
    return tlc


def coord_quiz(liste_pts, noms_pts):
    # Inicio
    quiz_quadrant = "$$\\fs3\\picture(361){"
    # Líneas horizontales
    quiz_quadrant += "(0,0){\\line(360,0)}(0,30){\\line(360,0)}(0,60){\\line(360,0)}(0,90){\\line(360,0)}(0,120){\\line(360,0)}(0,150){\\line(360,0)}(0,180){\\line(360,0)}(0,210){\\line(360,0)}(0,240){\\line(360,0)}(0,270){\\line(360,0)}(0,300){\\line(360,0)}(0,330){\\line(360,0)}(0,360){\\line(360,0)}"
    # Líneas Verticales
    quiz_quadrant += "(0,0){\\line(0,360)}(30,0){\\line(0,360)}(60,0){\\line(0,360)}(90,0){\\line(0,360)}(120,0){\\line(0,360)}(150,0){\line(0,360)}(180,0){\\line(0,360)}(210,0){\\line(0,360)}(240,0){\\line(0,360)}(270,0){\\line(0,360)}(300,0){\\line(0,360)}(330,0){\\line(0,360)}(360,0){\\line(0,360)}"
    # Ejes
    quiz_quadrant += "(0,179){\\line(360,0)}(0,181){\\line(360,0)}(179,0){\\line(0,360)}(181,0){\\line(0,360)}" 
    # Numeración eje OX
    quiz_quadrant += "(20,165){-5}(50,165){-4}(80,165){-3}(110,165){-2}(140,165){-1}(185,165){0}(215,165){1}(245,165){2}(275,165){3}(305,165){4}(335,165){5}"
    # Numeración eje OY
    quiz_quadrant += "(160,15){-5}(160,45){-4}(160,75){-3}(160,105){-2}(160,135){-1}(170,195){1}(170,225){2}(170,255){3}(170,285){4}(170,315){5}"
    # Puntos
    quiz_cor = ""
    for i in range(len(liste_pts)):
        liste_quiz = [180+30*liste_pts[i][0], 180+30*liste_pts[i][1]]
        quiz_quadrant += ("(%s,%s){\\bullet}(%s,%s){%s}" %(liste_quiz[0]-3, liste_quiz[1]-3, liste_quiz[0]+5, liste_quiz[1]+5, noms_pts[i]))
        quiz_cor += (_(u"$$%s$$ est %s <br/>")%(noms_pts[i],affiche_coord_quiz(liste_pts[i])))
    # Fin
    quiz_quadrant += "}$$"
    quiz_table = ("<table border=\"0\"><tbody><tr><td>%s</td><td>%s</td></tr></tbody></table>" % (quiz_quadrant, quiz_cor))
    return quiz_table


def main():
    nbpts = 13
    noms_pts=(noms_sommets(nbpts))
    coord_pts=coordo_pts(nbpts)
    voc=[_(u'abscisse'),_(u'ordonnée')]
    rg1=random.randrange(0,2)
    rg2=abs(rg1-1)
    while len(coord_pts)<nbpts:
        coord_pts=coordo_pts(nbpts)
    exo=["\\exercice",
         "\\parbox{0.4\\linewidth}{",
         "\\begin{enumerate}",
         _(u"\\item Donner les coordonnées des points %s, %s, %s, %s, %s et %s.") % tuple(noms_pts[0:6]),
         _(u"\\item Placer dans le repère les points %s, %s, %s, %s, %s et %s") %noms_pts[6:12] +_(u" de coordonnées respectives %s, %s, %s, %s, %s et %s. ") %tuple(tex_liste_co(coord_pts[6:12])),
         _(u"\\item Placer dans le repère le point %s d'%s %s et d'%s %s") %(noms_pts[12],voc[rg1],decimaux(str(coord_pts[12][rg1])),voc[rg2],decimaux(str(coord_pts[12][rg2]))),
         "\\end{enumerate}}\\hfill ",
         "\\parbox{0.55\\linewidth}{",
         "\\psset{unit=0.8cm}",
         "\\begin{pspicture}(-4.95,-4.95)(4.95,4.95)",
         "\\psgrid[subgriddiv=2, subgridcolor=lightgray, gridlabels=8pt](0,0)(-5,-5)(5,5)",
         "\\psline[linewidth=1.2pt]{->}(-5,0)(5,0)",
         "\\psline[linewidth=1.2pt]{->}(0,-5)(0,5)",
         place_points(coord_pts[0:6],noms_pts[0:6]),
         "\\end{pspicture}}"]
    cor=["\\exercice*",
         "\\parbox{0.4\\linewidth}{",
         "\\psset{unit=0.8cm}",
         "\\begin{pspicture}(-5,-5)(5,5)",
         "\\psgrid[subgriddiv=2, subgridcolor=lightgray, gridlabels=8pt](0,0)(-5,-5)(5,5)",
         "\\psline[linewidth=1.2pt]{->}(-5,0)(5,0)",
         "\\psline[linewidth=1.2pt]{->}(0,-5)(0,5)",
         place_points(coord_pts[0:13],noms_pts[0:13]),
         "\\end{pspicture}}\\hfill",
         "\\parbox{0.5\\linewidth}{",
         "\\begin{enumerate}",
         _(u"\\item Donner les coordonnées des points %s, %s, %s, %s, %s et %s.") % tuple(noms_pts[0:6])]
    quiz=["cloze"]
    i=0
    pts_quiz = []
    while i<6:
        cor.append(_(u"Les coordonnées du point %s sont %s \n")%(noms_pts[i],affiche_coord(coord_pts[i])))
        i=i+1
    cor[len(cor):len(cor)]=[_(u"\\item Placer dans le repère les points %s, %s, %s, %s, %s et %s") %noms_pts[6:12] +_(u" de coordonnées respectives %s, %s, %s, %s, %s et %s. ") %tuple(tex_liste_co(coord_pts[6:12])),
         _(u"\\item Placer dans le repère le point %s d'%s %s et d'%s %s") %(noms_pts[12],voc[rg1],decimaux(str(coord_pts[12][rg1])),voc[rg2],decimaux(str(coord_pts[12][rg2]))),
         "\\end{enumerate}"]
    cor.append("}")
    quiz_nom = _(u"Coordonnées de points")
    quiz_exo_cor = (_(u"Trouve les coordonnées des points $$%s$$, $$%s$$, $$%s$$, $$%s$$, $$%s\\,$$ et $$\\,%s$$<br/>\n") %tuple(noms_pts[0:6]))
    quiz_exo_cor += coord_quiz(coord_pts[0:6], noms_pts[0:6])
    quiz.append([quiz_nom, quiz_exo_cor, ""])
    return(exo,cor,quiz)

